/*
 *	Marvell ide driver.
 *
 * 	It's just a port of Alan Cox pata version.
 * 	Based on jmicron.c and pata_marvell.c
 *
 *	(c) 2006 Arnaud Patard  <apatard@mandriva.com>
 */

#include <linux/module.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/hdreg.h>
#include <linux/ide.h>
#include <linux/init.h>

#define DRV_NAME	"Marvell IDE driver"
#define DRV_VERSION	"20061120"

static u8 marvell_ratemask(ide_drive_t *drive)
{
	u8 mode = 4;

	if (!eighty_ninty_three(drive))
		mode = min(mode, (u8)1);

	return mode;
}

static int __devinit ata66_marvell(ide_hwif_t *hwif)
{
	struct pci_dev *pdev = hwif->pci_dev;
	u32 devices;
	void __iomem *barp;
	int i;
	int port = hwif->channel;

	/* Check if our port is enabled */

	barp = pci_iomap(pdev, 5, 0x10);
	if (barp == NULL)
		return -ENOMEM;
	printk("BAR5:");
	for(i = 0; i <= 0x0F; i++)
		printk("%02X:%02X ", i, readb(barp + i));
	printk("\n");

	devices = readl(barp + 0x0C);
	pci_iounmap(pdev, barp);

	if ((pdev->device == 0x6145) && (port == 0) &&
	    (!(devices & 0x10)))	/* PATA enable ? */
		return -ENOENT;

	/* Cable type */
	switch(port)
	{
	case 0:
		/* Might be backward, docs unclear */
		if (inb(hwif->dma_base + 1) & 1) {
			printk(KERN_INFO "%s: 40-pin cable\n", DRV_NAME);
			return 0;
		}
		else {
			printk(KERN_INFO "%s: 80-pin cable\n", DRV_NAME);
			return 1;
		}

	case 1: /* Legacy SATA port */
		printk(KERN_INFO "%s: sata cable\n", DRV_NAME);
		return 1;
	}

	return 0;
}

static void marvell_tune_proc (ide_drive_t *drive, byte mode_wanted)
{
        return;
}

static void config_marvell_chipset_for_pio (ide_drive_t *drive, byte set_speed)
{
	u8 speed = XFER_PIO_0 + ide_get_best_pio_mode(drive, 255, 5, NULL);

	if (set_speed)
		(void) ide_config_drive_speed(drive, speed);
}

static int marvell_tune_chipset (ide_drive_t *drive, byte xferspeed)
{
	u8 speed = ide_rate_filter(marvell_ratemask(drive), xferspeed);

	return ide_config_drive_speed(drive, speed);
}

static int config_chipset_for_dma (ide_drive_t *drive)
{
	u8 speed = ide_dma_speed(drive, marvell_ratemask(drive));

	config_marvell_chipset_for_pio(drive, !speed);
	marvell_tune_chipset(drive, speed);

	return ide_dma_enable(drive);
}

static int marvell_config_drive_for_dma (ide_drive_t *drive)
{
	if (ide_use_dma(drive) && config_chipset_for_dma(drive))
		return 0;

	config_marvell_chipset_for_pio(drive, 1);

	return -1;
}


static void __devinit init_hwif_marvell(ide_hwif_t *hwif)
{
	hwif->speedproc = &marvell_tune_chipset;
	hwif->tuneproc = &marvell_tune_proc;

	hwif->drives[0].autotune = 1;
	hwif->drives[1].autotune = 1;

	if (!hwif->dma_base)
		goto fallback;

	hwif->atapi_dma  = 1;
	hwif->ultra_mask = 0x7f;
	hwif->mwdma_mask = 0x07;

	hwif->ide_dma_check = &marvell_config_drive_for_dma;
	if (!(hwif->udma_four))
		hwif->udma_four = ata66_marvell(hwif);

	hwif->autodma = 1;
	hwif->drives[0].autodma = hwif->autodma;
	hwif->drives[1].autodma = hwif->autodma;
	return;
fallback:
	hwif->autodma = 0;
	return;
}

static ide_pci_device_t marvell_chipsets[] __devinitdata = {
	{
		.name		= "Marvell 88SE6101",
		.init_hwif	= init_hwif_marvell,
		.channels	= 1,
		.autodma	= AUTODMA,
		.bootable	= ON_BOARD,
	},
	{
		.name		= "Marvell 88SE6145",
		.init_hwif	= init_hwif_marvell,
		.channels	= 2,
		.autodma	= AUTODMA,
		.bootable	= ON_BOARD,
	}
};

/*
 *	marvell_init_one - Register Marvell ATA PCI device with kernel services
 */
static int marvell_init_one (struct pci_dev *pdev, const struct pci_device_id *id)
{
	ide_setup_pci_device(pdev, &marvell_chipsets[id->driver_data]);
	return 0;
}

static const struct pci_device_id marvell_pci_tbl[] = {
	{ 0x11AB, 0x6101, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{ 0x11AB, 0x6145, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 1},
	{ }	/* terminate list */
};

static struct pci_driver marvell_pci_driver = {
	.name			= DRV_NAME,
	.id_table		= marvell_pci_tbl,
	.probe			= marvell_init_one,
};

static int __init marvell_init(void)
{
	return ide_pci_register_driver(&marvell_pci_driver);
}

module_init(marvell_init);

MODULE_AUTHOR("Arnaud Patard");
MODULE_DESCRIPTION("IDE low-level driver for Marvell ATA in legacy mode");
MODULE_LICENSE("GPL");
MODULE_DEVICE_TABLE(pci, marvell_pci_tbl);
MODULE_VERSION(DRV_VERSION);

